import 'dart:math';
import 'package:flutter/foundation.dart';
import '../uibit.dart';
import 'package:vector_math/vector_math.dart' as vector;

typedef DisplayChange = void Function(bool isOpen);

class FabCircularMenu extends StatefulWidget {
  final List<Widget> children;
  final double fabSize;
  final double fabElevation;
  final EdgeInsets fabMargin;
  final DisplayChange onDisplayChange;

  FabCircularMenu({
    Key key,
    this.fabSize = 50.0,
    this.fabElevation = 8.0,
    this.fabMargin = const EdgeInsets.all(15.0),
    this.onDisplayChange,
    @required this.children,
  }) : super(key: key);

  @override
  FabCircularMenuState createState() => FabCircularMenuState();
}

class FabCircularMenuState extends State<FabCircularMenu> with SingleTickerProviderStateMixin {
  double _marginH = 0;
  double _marginV = 0;
  double _directionX = 1;
  double _directionY = 1;
  double _translationX = 0;
  double _translationY = 0;

  double get _ringDiameter => min(kSize.width, kSize.height) * 1.25;

  AnimationController _animationController;
  Animation<double> _scaleAnimation;
  Animation _scaleCurve;
  Animation<double> _rotateAnimation;
  Animation _rotateCurve;

  bool _isOpen = false;
  bool _isAnimating = false;

  @override
  void initState() {
    super.initState();

    _animationController = AnimationController(duration: Duration(milliseconds: 600), vsync: this);

    _animationController.addListener(() {
      setState(() {});
    });

    _scaleCurve = CurvedAnimation(parent: _animationController, curve: Interval(0.0, 0.4, curve: Curves.fastOutSlowIn));
    _scaleAnimation = Tween<double>(begin: 0.0, end: 1.0).animate(_scaleCurve);

    _rotateCurve = CurvedAnimation(parent: _animationController, curve: Interval(0.4, 1.0, curve: Curves.easeInOutCirc));
    _rotateAnimation = Tween<double>(begin: 0.5, end: 1.0).animate(_rotateCurve);
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _calculateProps();
  }

  @override
  Widget build(BuildContext context) {
    // This makes the widget able to correctly redraw on
    // hot reload while keeping performance in production
    if (!kReleaseMode) {
      _calculateProps();
    }

    return Container(
      child: Stack(
        alignment: Alignment.bottomRight,
        children: <Widget>[
          _scaleAnimation.value == 1.0
              ? InkWell(
                  onTap: () {
                    close();
                  },
                  child: Container(
                    width: kSize.width,
                    height: kSize.height,
                    color: kColor.dark3,
                  ),
                )
              : Center(),
          // Ring
          Transform(
            transform: Matrix4.translationValues(_translationX, _translationY, 0.0)..scale(_scaleAnimation.value),
            alignment: FractionalOffset.center,
            child: OverflowBox(
              maxWidth: _ringDiameter,
              maxHeight: _ringDiameter,
              child: Container(
                width: _ringDiameter,
                height: _ringDiameter,
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(_ringDiameter),
                    border: Border.all(
                      width: 2,
                      color: kColor.light5,
                    )),
                child: BlurMask(
                  borderRadius: BorderRadius.circular(_ringDiameter),
                  child: _scaleAnimation.value == 1.0
                      ? Transform.rotate(
                          angle: ((2 * pi) * _rotateAnimation.value * _directionX * _directionY).toDouble(),
                          child: Container(
                            child: Stack(
                              alignment: Alignment.center,
                              children: widget.children.asMap().map((index, child) => MapEntry(index, _applyTransformations(child, index))).values.toList(),
                            ),
                          ),
                        )
                      : Container(),
                ),
              ),
            ),
          ),

          // FAB
          Container(
            height: 60,
            width: 60,
            margin: kEdge.all,
            child: Button.icon(
              round: kColor.mind,
              rounded: true,
              icon: _scaleAnimation.value == 1.0 ? Icons.close : Icons.album,
              onPressed: () {
                if (_isAnimating) return;
                if (_isOpen) {
                  close();
                } else {
                  open();
                }
              },
            ),
          ),
        ],
      ),
    );
  }

  Widget _applyTransformations(Widget child, int index) {
    double angleFix = 0.0;
    final angle = vector.radians(90.0 / (widget.children.length - 1) * index + angleFix);
    return Transform(
        transform: Matrix4.translationValues((-(_ringDiameter / 2) * cos(angle) + (_ringDiameter * 0.3 / 2 * cos(angle))) * _directionX, (-(_ringDiameter / 2) * sin(angle) + (_ringDiameter * 0.3 / 2 * sin(angle))) * _directionY, 0.0),
        alignment: FractionalOffset.center,
        child: Material(
          color: Colors.transparent,
          child: Padding(
            padding: const EdgeInsets.all(15.0),
            child: child,
          ),
        ));
  }

  _calculateProps() {
    _marginH = (widget.fabMargin.right + widget.fabMargin.left) / 2;
    _marginV = (widget.fabMargin.top + widget.fabMargin.bottom) / 2;
    _translationX = ((kSize.width - widget.fabSize) / 2 - _marginH);
    _translationY = ((kSize.height - widget.fabSize) / 2 - _marginV);
  }

  open() {
    _isAnimating = true;
    _animationController.forward().then((_) {
      _isAnimating = false;
      _isOpen = true;
      if (widget.onDisplayChange != null) {
        widget.onDisplayChange(true);
      }
    });
  }

  close() {
    _isAnimating = true;
    _animationController.reverse().then((_) {
      _isAnimating = false;
      _isOpen = false;
      if (widget.onDisplayChange != null) {
        widget.onDisplayChange(false);
      }
    });
  }

  bool get isOpen => _isOpen;
}
