import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';

class InkTapController {
  final GestureTapCallback onTap;
  final GestureTapDownCallback onTapDown;
  final GestureTapCallback onTapCancel;
  final GestureTapCallback onDoubleTap;
  final GestureLongPressCallback onLongPress;
  final ValueChanged<bool> onHighlightChanged;
  final ValueChanged<bool> onHover;
  final ValueChanged<bool> onFocusChange;

  final bool enableFeedback;
  InkTapController({
    this.onTap,
    this.onTapDown,
    this.onTapCancel,
    this.onDoubleTap,
    this.onLongPress,
    this.onHighlightChanged,
    this.onFocusChange,
    this.onHover,
    this.enableFeedback = true,
  });
}


class SwipeConfig {
  double verticalSwipeMinDisplacement = 25.0;
  double verticalSwipeMinVelocity = 350.0;

  double horizontalSwipeMinDisplacement = 25.0;
  double horizontalSwipeMinVelocity = 350.0;

  SwipeConfig({
    double verticalSwipeMaxWidthThreshold,
    double verticalSwipeMinDisplacement,
    double verticalSwipeMinVelocity,
    double horizontalSwipeMaxHeightThreshold,
    double horizontalSwipeMinDisplacement,
    double horizontalSwipeMinVelocity,
  }) {
    if (verticalSwipeMinDisplacement != null) {
      this.verticalSwipeMinDisplacement = verticalSwipeMinDisplacement;
    }

    if (verticalSwipeMinVelocity != null) {
      this.verticalSwipeMinVelocity = verticalSwipeMinVelocity;
    }

    if (horizontalSwipeMinDisplacement != null) {
      this.horizontalSwipeMinDisplacement = horizontalSwipeMinDisplacement;
    }

    if (horizontalSwipeMinVelocity != null) {
      this.horizontalSwipeMinVelocity = horizontalSwipeMinVelocity;
    }
  }
}

class SwipeDetector extends StatefulWidget {
  final Widget child;
  final SwipeController swipeController;

  const SwipeDetector({Key key, this.child, this.swipeController}) : super(key: key);

  @override
  State<StatefulWidget> createState() => SwipeDetectorState();
}

class SwipeDetectorState extends State<SwipeDetector> {
  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      behavior: HitTestBehavior.opaque,
      onDoubleTap: () {
        if (widget.swipeController.onDoubleTap != null) widget.swipeController.onDoubleTap();
      },
      onVerticalDragStart: widget.swipeController.onVerticalDragStart,
      onVerticalDragUpdate: widget.swipeController.onVerticalDragUpdate,
      onVerticalDragEnd: widget.swipeController.onVerticalDragEnd,
      onHorizontalDragStart: widget.swipeController.onHorizontalDragStart,
      onHorizontalDragUpdate: widget.swipeController.onHorizontalDragUpdate,
      onHorizontalDragEnd: widget.swipeController.onHorizontalDragEnd,
      child: widget.child,
    );
  }
}

class SwipeController {
  final Function() onSwipeUp;
  final Function() onSwipeDown;
  final Function() onSwipeLeft;
  final Function() onSwipeRight;
  final Function() onDoubleTap;
  final Function(DragEndDetails) onSwipeEnd;
  final Function(DragUpdateDetails) onSwipeUpdateHorizontal;
  final Function(DragUpdateDetails) onSwipeUpdateVertical;

  SwipeConfig swipeConfig = SwipeConfig();

  //Vertical drag details
  DragStartDetails startV;
  DragUpdateDetails updateV;

  //Horizontal drag details
  DragStartDetails startH;
  DragUpdateDetails updateH;

  SwipeController({this.onSwipeUp, this.onSwipeDown, this.onSwipeLeft, this.onSwipeRight, this.onDoubleTap, this.onSwipeEnd, this.onSwipeUpdateHorizontal, this.onSwipeUpdateVertical});

  onHorizontalDragStart(DragStartDetails details) {
    startH = details;
  }

  onHorizontalDragUpdate(DragUpdateDetails details) {
    updateH = details;
    if (onSwipeUpdateVertical != null) onSwipeUpdateVertical(details);
  }

  onHorizontalDragEnd(DragEndDetails details) {
    if (onSwipeEnd != null) onSwipeEnd(details);
    if (startH == null || updateH == null) return;
    double dx = updateH.localPosition.dx - startH.localPosition.dx;
    double dy = updateH.localPosition.dy - startH.localPosition.dy;
    double velocity = details.primaryVelocity;
    if (dx < 0) dx = -dx;
    if (dy < 0) dy = -dy;
    double positiveVelocity = velocity < 0 ? -velocity : velocity;
    if (dx < swipeConfig.horizontalSwipeMinDisplacement) return;
    if (positiveVelocity < swipeConfig.horizontalSwipeMinVelocity) return;

    if (velocity < 0) {
      if (onSwipeLeft != null) onSwipeLeft();
    } else {
      if (onSwipeRight != null) onSwipeRight();
    }
  }

  onVerticalDragStart(DragStartDetails details) {
    startV = details;
  }

  onVerticalDragUpdate(DragUpdateDetails details) {
    updateV = details;
    if (onSwipeUpdateHorizontal != null) onSwipeUpdateHorizontal(details);
  }

  onVerticalDragEnd(DragEndDetails details) {
    if (onSwipeEnd != null) onSwipeEnd(details);
    if (startV == null || updateV == null) return;
    double dy = updateV.localPosition.dy - startV.localPosition.dy;
    double velocity = details.primaryVelocity;
    if (dy < 0) dy = -dy;
    double positiveVelocity = velocity < 0 ? -velocity : velocity;
    if (dy < swipeConfig.verticalSwipeMinDisplacement) return;
    if (positiveVelocity < swipeConfig.verticalSwipeMinVelocity) return;
    if (velocity < 0) {
      if (onSwipeUp != null) onSwipeUp();
    } else {
      if (onSwipeDown != null) onSwipeDown();
    }
  }
}


